#!/usr/bin/python
#-*- coding: utf8 -*-

import gettext
import threading
import gtk
import gtk.glade
import subprocess
import os
import sys
import gtk
import tempfile
import string
from shutil import copy, move
from config import *
import preference as preference
import gobject
import re

_ = gettext.gettext

def run_process(command):
	f = subprocess.Popen(
		command, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
		stderr=subprocess.PIPE, close_fds=True, shell=True)
	return f.stdout

def print_log(text, block=True):
	textview = glade.get_widget('textview1')
	buffer = textview.get_buffer()
	#buffer.delete(buffer.get_start_iter(),buffer.get_end_iter())
	if block:
		gtk.gdk.threads_enter()
		buffer.insert(buffer.get_end_iter(), text + '\n')
		match_start_mark = buffer.create_mark(
					'match_start', buffer.get_end_iter(), True)
		textview.scroll_to_mark(match_start_mark, 0, True)
		gtk.gdk.threads_leave()
	else:
		buffer.insert(buffer.get_end_iter(), text + '\n')
		match_start_mark = buffer.create_mark(
					'match_start', buffer.get_end_iter(), True)
		textview.scroll_to_mark(match_start_mark, 0, True)

def on_preference_activate(widget):
	preference.preference_activate()

def codecs_list(glade):
	codecs = ['ogg', 'mp3', 'flac', 'wav', 'ape', 'wv']
	list = glade.get_widget('codec').get_model()
	list.clear()
	map(lambda x: list.append([x]), codecs)
	st = get('codec')
	glade.get_widget('codec').set_active(st)

def varc(glade1, pathglade1, QUIET):
	global glade, pathglade, QUIET1
	glade = glade1
	pathglade = pathglade1
	QUIET1 = QUIET

def parametrs(glade):
	dic = {
		'performer': glade.get_widget('performer').get_text(),
		'bitrate': str(int(glade.get_widget('bitrate').get_value())),
		'quality': glade.get_widget('quality').get_text(),
		'album': glade.get_widget('album').get_text(),
		'date': glade.get_widget('date').get_text(),
		'genre': glade.get_widget('genre').get_text(),
		'composer': glade.get_widget('composer').get_text(),
		'level': glade.get_widget('level').get_active_text(),
		'level_n': glade.get_widget('level').get_active(),
		'bitratebtn': glade.get_widget('bitratebtn').get_active(),
		'qualitybtn': glade.get_widget('qualitybtn').get_active(),
		'VBR': glade.get_widget('VBR').get_active(),
		'bin_file': glade.get_widget('bin_file').get_text(),
		'filename': glade.get_widget('filename').get_text(),
		'codec': glade.get_widget('codec').get_active_text(),
		'codec_n': glade.get_widget('codec').get_active(),
		'folder': glade.get_widget('savefolder').get_text(),
		'same_folder': glade.get_widget('same_folder').get_active(),
		}
	return dic

def show_dialog(message):
	try: message = message.replace('<b>', '').replace('</b>', '')
	except: pass
	print message
	print_log(message, False)
	dialog = gtk.MessageDialog(parent=None,
			flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING,
			buttons=gtk.BUTTONS_OK)
	dialog.set_markup(message)
	gobject.timeout_add(10000, lambda: dialog.destroy())
	response = dialog.run()
	if not os.path.exists(get('tmpdir')):
		dialog.destroy()
		return
	clean_tmp(get('tmpdir'))
	dialog.destroy()

def set_text_progressbar1(text):
	gtk.gdk.threads_enter()
	glade.get_widget('progressbar1').set_text(text)
	preference.progressbar1.set_text(text)
	gtk.gdk.threads_leave()

def	set_fraction_progressbar1(fraction):
	gtk.gdk.threads_enter()
	glade.get_widget('progressbar1').set_fraction(fraction)
	preference.progressbar1.set_fraction(fraction)
	gtk.gdk.threads_leave()

def start():

	def find_bin(path, fname):
		extention = ('ape', 'flac', 'wv', 'flake', 'wav', 'tta')
		fname = fname.split('.')[-2]
		for ext in extention:
			if not os.path.isdir(path):
				return
			for name in os.listdir(path):
				path_name = os.path.join(path, name)
				if path_name.endswith(ext) and (name.split('.')[-2] == fname):
					return path_name

	def codec_path(codec):
		path = get(codec)
		if not os.path.isfile(path):
			gtk.gdk.threads_enter()
			message = _('Codec\n <b>%s</b>\nmissing') %path
			show_dialog(message)
			glade.get_widget('stop_button').set_sensitive(False)
			gtk.gdk.threads_leave()
		else:
			return True

	def file_not_found():
		gtk.gdk.threads_enter()
		message = (_('File \n<b>%s</b>\nnot found') %bin_file)
		show_dialog(message)
		gtk.gdk.threads_leave()

	if not os.path.isdir(get('tmp_dir')):
		gtk.gdk.threads_enter()
		message = (_('Temp dir \n<b>%s</b>\nnot found') %get('tmp_dir'))
		show_dialog(message)
		glade.get_widget('stop_button').set_sensitive(False)
		gtk.gdk.threads_leave()
		return

	treeview = glade.get_widget('job queue')
	model = treeview.get_model()
	dic = model[0][6]
	folder = os.path.dirname(dic['filename'])
	all_tr = model.iter_n_children(model.get_iter(0))
	'''list tracks for decoder'''
	rowl = ''
	for n in range(model.iter_n_children(model.get_iter(0))):
		if model[0, n][3]:
			rowl += str(model[0, n][0]) + ','
	dic['trlist'] = rowl[:-1]
	dic['all_tr'] = all_tr
	for n in range(all_tr):
		if model[0, n][3]:
			dic[n+1] = model[0, n][1]
	clear_root_iter(glade)
	Conversation.job_name = '%s - %s' %(dic['performer'], dic['album'])
	gtk.gdk.threads_enter()
	glade.get_widget('stop_button').set_sensitive(True)
	progressbar2=glade.get_widget('progressbar2')
	progressbar2.set_fraction(0)
	preference.progressbar2.set_fraction(0)
	preference.progressbar2_text = '%s   in queue:%s' %(
											Conversation.job_name, len(model))
	progressbar2.set_text(preference.progressbar2_text)
	preference.progressbar2.set_text(preference.progressbar2_text)
	gtk.gdk.threads_leave()
	set_fraction_progressbar1(0)
	'''decode to tmpdir'''
	tmpdir = tempfile.mkdtemp(prefix='gcue2tracks', dir=get('tmp_dir'))
	set('tmpdir', tmpdir)
	if 'bin_files' in dic:
		'''tracks + cue'''
		for n in rowl.split(','):
			if n is '':
				break
			filename = dic['bin_files'][int(n) - 1]
			bin_file = os.path.join(folder, filename)
			if not os.path.isfile(bin_file):
				extention = ('ape', 'flac', 'wv', 'flake', 'wav', 'tta')
				if not os.path.isdir(folder):
					return
				for ext in extention:
					for name in os.listdir(folder):
						path_name = os.path.join(folder, name)
						if path_name.endswith(ext) or not ext:
							if filename.split('.')[0] in name.split('.')[0]:
								bin_file = path_name
								break
				if not bin_file:
					file_not_found()
					return
			command = "ffmpeg -y -i '%s' '%s/split-track%s.wav'" % (bin_file, tmpdir, n)
			text = _('Decoding to wav "%s" (%d/%s) ') \
				%(dic[int(n)], int(n), all_tr)
			decode_to_wav(command.encode('utf-8'), dic, text, False)
			print_log(text)
			gtk.gdk.threads_enter()
			progressbar2 = glade.get_widget('progressbar2')
			frac = progressbar2.get_fraction() +\
				2*(1.0/(2 + len(dic['trlist'].split(','))))\
				/len(dic['trlist'].split(','))
			progressbar2.set_fraction(frac)
			preference.progressbar2.set_fraction(frac)
			gtk.gdk.threads_leave()
	else:
		f = run_process('cuebreakpoints "%s" > %s/cuebreakpoints' %
			(dic['filename'], tmpdir))
		bin_file = os.path.join(folder, dic['bin_file'])
		'''file exits?'''
		if not os.path.isfile(bin_file):
			file_not_found()
			return
		print_log(_('Decoding to wav '))
		command = 'ffmpeg -y -i "%s" "%s/wav.wav"' % (bin_file, tmpdir)
		decode_to_wav(command, dic, text = _('Decoding to wav '))

		'''split tracks'''
		if Conversation.value == (_('Interrupted')):
			clean_tmp(tmpdir)
			return
		if Conversation.value == 'error':
			print 'decode error'
			print_log('decode error', False)
			clean_tmp(tmpdir)
			return
		print_log(_('split tracks  '))
		command = ('shnsplit -x "%s" '
				'-o "wav ext=wav shntool" '
				'-d "%s" '
				'-f "%s/cuebreakpoints" %s/wav.wav' %
				(dic['trlist'], tmpdir, tmpdir, tmpdir))
		process(command.encode('utf-8'), dic, True,
									text = _('split tracks  '))

	'''encoding tracks'''
	encoder(tmpdir, dic)

	'''movie encoded files'''
	a = Copy_result(tmpdir, dic)
	a = a.run()

def track_light(path, filename, glade):
	'''get track light'''
	filename = filename.encode('utf-8')
	path = path.encode('utf-8')
	os.chdir(path)
	f = run_process(
		'cuebreakpoints "%s"' % glade.get_widget('filename').get_text())
	fromchild = f.readline()
	list = []
	start = 0
	len1 = ''
	while fromchild:
		fromchild = fromchild.replace('\n', '')
		start, len1 = pars(fromchild, start, len1)
		list.append(len1)
		fromchild = f.readline()
	f = run_process('shnlen -q "%s/%s"' % (path, filename))
	fromchild = f.readlines()
	light = fromchild[1].strip().split(' ')[0]
	start, len1 = pars(light, start, len1)
	list.append(len1)
	return list, light

def pars(fromchild, start, len1):
	a1=string.find(fromchild, ':')
	if a1 != -1:
		min = int(fromchild[:a1])
		sec = int(fromchild[(a1+1):(a1+3)])
		dif = min*60+sec-start
		secs = str(dif-((dif/60)*60))
		if len(secs) < 2:
			secs = '0' + secs
		len1 = '%s:%s'% (str(dif/60), secs)
		start = min*60+sec
	return start, len1

def col0_edited_cb(cell, path, new_text, model):
	#print "Change '%s' to '%s'" % (model[path][1], new_text)
	model[path][1] = new_text
	return

def col1_toggled_cb(cell, path, model):
	a = model[path][3] = not model[path][3]
	treeview = glade.get_widget('job queue')
	iter = model.get_iter(path)
	s = model.iter_depth(iter)
	if s is not 0:
		return
	for n in range(model.iter_n_children(model.get_iter(path[0]))):
		model[int(path[0]), n][3] = a

def clean_tmp(tmpdir):
	for name in os.listdir(tmpdir):
		path_name = os.path.join(tmpdir, name)
		os.remove(path_name)
	os.rmdir(tmpdir)

def headerClick(TreeViewColumn, Column):
	print 'click', Column


def clear_root_iter(glade):
	treeview = glade.get_widget('job queue')
	model = treeview.get_model()
	iter_root = model.get_iter_root()
	if iter_root is None:
		return
	gtk.gdk.threads_enter()
	model.remove(iter_root)
	if len(model) ==0:
		glade.get_widget('del_task').set_sensitive(False)
	else:
		treeview.set_cursor(0)
	if len(model) == 1:
		glade.get_widget('job_up').set_sensitive(False)
		glade.get_widget('job_down').set_sensitive(False)
	gtk.gdk.threads_leave()

def getvar(file_name, dic, trac_number):
	level = dic['level']
	codec = dic['codec']
	title = dic[int(trac_number)]
	if 'performers' in dic:
		performer = dic['performers'][int(trac_number)]
	else:performer = dic['performer']

	if codec =='flac':
		arg = '%s -T TITLE="%s" -T ALBUM="%s" -T TRACKNUMBER=%s \
		-T ARTIST="%s" -T GENRE="%s" -T DATE="%s" \
		-T DESCRIPTION="gCue2tracks" -T COMPOSER="%s" ' % (
		get('flac'), title, dic['album'], trac_number,
		performer, dic['genre'],
		dic['date'], dic['composer'])
		if level =='mid':
			arg += '-5 '
		else:
			arg += '--%s ' %level

	if codec =='ogg':
		arg = '%s -a "%s" -d "%s" -G "%s" -l "%s" -N "%s" -t "%s" -c "%s"' % (
				get('ogg'), performer, dic['date'], dic['genre'],
				 dic['album'], trac_number, title, 'DESCRIPTION=gCue2tracks')

		if dic['bitratebtn']:
			if dic['VBR']:
				arg += ' -b %s ' %dic['bitrate']
			else:
				arg += ' --managed -b %s -M %s ' % (dic['bitrate'], dic['bitrate'])
		else:
			arg += ' -q %s ' %dic['quality']

	if codec =='mp3':
		arg = '%s --id3v2-only --ta "%s" --tt "%s" --tl "%s" --ty "%s" --tc "%s" \
			--tn "%s/%s" --tg "%s" --nohist' % (get('mp3'),
			performer, title, dic['album'], dic['date'], 'gCue2tracks',
			trac_number, dic['all_tr'], dic['genre'])
		if get('mp3advance'):
			if dic['bitratebtn']:
				if dic['VBR']:
					arg += ' -v -B %s -q %s ' % (dic['bitrate'], dic['quality'])
				else:
					arg += ' -b %s -q %s ' % (dic['bitrate'], dic['quality'])
		else:
			if dic['bitratebtn']:
				if dic['VBR']:
					arg += ' -v -B %s ' %dic['bitrate']
				else:
					arg += ' -b %s ' %dic['bitrate']
			else:
				arg += ' -q %s ' %dic['quality']

	if codec =='wv':
		arg = '%s -w "Title=%s" -w "Artist=%s" -w "Album=%s" '\
			'-w "Track=%s/%s" -w "Composer=%s" -w "Comment=gCue2tracks" '\
			'-w "Year=%s" -w "Genre=%s" ' % (
			get('wavpack'), title, performer, dic['album'],
			trac_number, dic['all_tr'], dic['composer'],
			dic['date'], dic['genre'])
		if level =='fast':
			arg +='-f '
		if level =='best':
			arg +='-hh '
		if level =='mid':
			arg +='-h '

	if codec =='ape':
		arg = '%s %s %s ' % (get('mac'), file_name, file_name + '.ape')
		if level =='fast':
			arg +='-c1000 '
		if level =='best':
			arg +='-c4000 '
		if level =='mid':
			arg +='-c2000 '
		return arg

	arg += file_name
	return arg

def encoder(tmpdir, dic):
	codec = dic['codec']
	if codec =='wav':
		return
	if Conversation.value == (_('Interrupted')):
		return
	if Conversation.value == 'error':
		print 'split error'
		print_log('split error', False)
		return
	tmp_file_list = []
	tr_list = dic['trlist'].split(',')
	tr_to_enc = str(len(tr_list))
	print 'start encode'
	print_log('start encode')

	# search tracks
	for name in os.listdir(tmpdir):
		if name.startswith('split-track'):
			file_name = os.path.join(tmpdir, name)
			tmp_file_list.append(file_name)
	tmp_file_list.sort()

	n = 0
	for file_name in tmp_file_list:
		trac_number = tr_list[n]
		title = dic[int(trac_number)]
		text = _('Encoding "%s" (%s/%s)  ') % (title, str(n+1), tr_to_enc)
		print text
		print_log(text)
		command = getvar(file_name, dic, trac_number).replace('`', '\`')
		print command
		print_log(command)
		if codec =='wv':
			process(command, dic, text = text)
		process(command, dic, True, text = text)
		if Conversation.value == 'error' or Conversation.value == (_('Interrupted')):
			return
		n +=1

def process(command, dic, progr = None, text = ''):

	def ss(a, b, n, fraction):
		b += a
		if a == '%':
			c = (b[-7:-1]).split(' ')
			try:
				c[-1] = c[-1].replace(',', '.').replace('[', '').replace('(', '')
			except:
				pass
			try:
				n = float(c[-1])
			except:
				pass
			b = '0000000'

		progres = n/100.0
		if progres > fraction:
			set_fraction_progressbar1(progres)
			prog_text = '%s%s%s' %(text, str(n), '%')
			set_text_progressbar1(prog_text)
			#preference.progressbar2.set_text(preference.progressbar2_text)
			fraction = progres
		if progres < fraction:
			set_fraction_progressbar1(0)
			fraction = 0
		return b, n, fraction


	if Conversation.value == (_('Interrupted')):
		return
	if Conversation.value == 'error':
		print ' error'
		return
	fraction = 0
	all_tr = dic['all_tr']
	tr_to_enc = len(dic['trlist'].split(','))
	set_text_progressbar1(text)
	progressbar2 = glade.get_widget('progressbar2')
	try:
		Conversation.value = 'in process'
		proc1 = subprocess.Popen(command, stdin=subprocess.PIPE,
				stdout=subprocess.PIPE, stderr=subprocess.PIPE,
				close_fds=True, shell=True)
		Conversation.pid = proc1.pid
		n = 0
		b = '0000000'
		while proc1.poll() == None:
			b, n, fraction = ss(proc1.stderr.read(1), b, n, fraction)
		while proc1.stderr.read(1):
			b, n, fraction = ss(proc1.stderr.read(1), b, n, fraction)
		if proc1.poll() not in [256, 0, -1]:
			set_fraction_progressbar1(0.0)
			if proc1.poll() == 9:
				set_text_progressbar1(_('Interrupted'))
				Conversation.value = (_('Interrupted'))
			else:
				set_text_progressbar1(_('Error'))
				Conversation.value = 'error'
		elif proc1.poll()==0:
			if progr:
				gtk.gdk.threads_enter()
				frac = progressbar2.get_fraction() +(1.0/(2 + tr_to_enc))
				progressbar2.set_fraction(frac)
				preference.progressbar2.set_fraction(frac)
				gtk.gdk.threads_leave()
			set_fraction_progressbar1(0)
			Conversation.value = 'complete'
	except:
		preference.trayicon.set_tooltip('error')
		Conversation.value = 'error'
		print 'error in ' + self.getName() + command
		print_log('error in ' + self.getName() + command)

def decode_to_wav(command, dic, text='', use_pbar=True):
	if Conversation.value == (_('Interrupted')):
		return
	if Conversation.value == 'error':
		print ' error'
		return
	fraction = 0
	all_tr = dic['all_tr']
	tr_to_enc = len(dic['trlist'].split(','))
	set_text_progressbar1(text)
	progressbar2 = glade.get_widget('progressbar2')
	Conversation.value = 'in process'
	proc1 = subprocess.Popen(command, stdin=subprocess.PIPE,
			stdout=subprocess.PIPE, stderr=subprocess.PIPE,
			close_fds=True, shell=True)
	Conversation.pid = proc1.pid
	a = ''
	while proc1.poll() == None:
		b = proc1.stderr.read(1)
		if b == '\r':
			#a =  'Duration: N/A, bitrate: N/A'
			u = re.search("Duration: (?P<name>.*?),",a)
			if u:
				u = u.group('name').split(':')
				if u[0] == 'N/A':
					duration = 0
					break
				duration = (int(u[0]) * 60 + int(u[1])) * 60 + \
													int(u[2].split('.')[0])
				break
		a += b
	while proc1.poll() == None:
		b = proc1.stderr.read(1)
		if b == '\r':
			u = re.search('time\s{0,}=\s{0,}(?P<time>\d+\.\d+)\s{1,}bitrate', a)
			if u:
				time = float(u.group('time').split('.')[0])
				if duration == 0:
					glade.get_widget('progressbar1').pulse()
				else:
					progres = time/duration
					set_fraction_progressbar1(progres)
					prog_text = '%s%s%s' %(text, str(int(progres*100)), '%')
					set_text_progressbar1(prog_text)
				a = ""
		a += b
	if proc1.poll() not in [256, 0, -1]:
		set_fraction_progressbar1(0.0)
		if proc1.poll() == 9:
			set_text_progressbar1(_('Interrupted'))
			Conversation.value = (_('Interrupted'))
		else:
			set_text_progressbar1(_('Error'))
			Conversation.value = 'error'
	elif proc1.poll()==0:
		if use_pbar:
			gtk.gdk.threads_enter()
			frac = progressbar2.get_fraction() +(1.0/(2 + tr_to_enc))
			progressbar2.set_fraction(frac)
			preference.progressbar2.set_fraction(frac)
			gtk.gdk.threads_leave()
		set_fraction_progressbar1(0)
		Conversation.value = 'complete'

class Copy_result():

	def __init__(self, tmpdir, dic):
		self.tmpdir = tmpdir
		self.dic = dic

	def run(self):
		if Conversation.value == (_('Interrupted')):
			clean_tmp(self.tmpdir)
			return
		if Conversation.value == 'error':
			print 'cleaning temp'
			print_log('cleaning temp')
			clean_tmp(self.tmpdir)
			return
		print 'start copy file'
		print_log('start copy file')
		if self.dic['codec'] =='mp3' and get('mp3_utf'):
			# 'python-mutagen'
			set_text_progressbar1(_('Convert tag to UTF-8'))
			for root, dirs, files in os.walk(self.tmpdir):
				for f in files:
					if f.endswith('mp3'):
						conv = 'mid3iconv -e UTF-8 "%s/%s"' % (root, f)
						proc1 = subprocess.Popen(conv, shell=True)
						os.waitpid(proc1.pid, 0)

		dic = {
			'%A': self.dic['album'],
			'%a': 'Album disc number',
			'%P': self.dic['performer'],
			'%D': self.dic['date'],
			'%G': self.dic['genre'],
			'%t': 'Track title',
			'%p': self.dic['performer'],
			'%g': self.dic['genre'],
			'%n': '%name',
			'%N': 'Track number'}
		if self.dic['date'] is '':
			dic['%D'] = 'unknown'
		format = get('format')
		for n in dic:
			try:
				format = format.replace(n, dic[n])
			except:
				pass
		newdir = format[:format.rfind('/')]
		try:
			set_text_progressbar1(_('Create folder'))
			newdir = self.dic['folder'] + newdir
			os.makedirs(newdir)
		except OSError, (errno, strerror):
			if errno == 13:
				print 'OSError(%s): %s' % (errno, strerror)
				print_log('OSError(%s): %s   can\'t create new dir "%s"' % (
													errno, strerror, newdir))
				clean_tmp(self.tmpdir)
				return

		newname_ = format[format.rfind('/') + 1:]
		tr_list = self.dic['trlist'].split(',')
		tmp_file_list = []
		for name in os.listdir(self.tmpdir):
			if name.endswith(self.dic['codec']) and name.startswith('split'):
				tmp_file_list.append(name)
		tmp_file_list.sort()

		os.chdir(self.tmpdir)
		n = 0
		filtr = re.compile(r"[?\\*:><\"\|]") # filename filter ?  \ * : >< " |
		filename_filter = get('filename_filter')
		for file_name in tmp_file_list:
			self.trac_number = tr_list[n]
			self.title = self.dic[int(self.trac_number)]
			try:
				newname = newname_.replace('Track title', self.title)
				newname = newname.replace('%name', str(int(self.trac_number)))
			except:
				pass
			try:
				newname = newname.replace('Track number', self.trac_number)
			except:
				pass
			newname += '.%s' %self.dic['codec']
			# replace ? / \ * : >< " | in filenames
			if '/' in newname:
				newname = newname.replace('/', '_')
			if filename_filter:
				newname = filtr.sub('_', newname)

			os.rename(file_name, newname)
			set_text_progressbar1(_('Taging and copy %s' %newname))
			print newname, '=>> ' + newdir
			print_log(newname, '=>> ' + newdir)
			move(newname, newdir)
			n += 1

		if get('copy_image') and self.dic['filename'] != newdir:
			'''copy image files '''
			set_text_progressbar1(_('Copy image files '))
			import mimetypes
			copy_from = os.path.dirname(self.dic['filename'])
			mimetypes.init()
			for root, dirs, files in os.walk(copy_from):
				for name in dirs:
					if name not in newdir:
						try:
							os.mkdir(os.path.join(newdir, name))
						except OSError, (errno, strerror):
							pass#print 'OSError(%s): %s (%s)' \
							#% (errno, strerror, os.path.join(newdir, name))
				for name in files:
					filetype = mimetypes.guess_type(name)[0]
					if type(filetype) is str:
						filetype = filetype.split('/')[0]
						if filetype =='image':
							try:
								copy(
										os.path.join(root, name),
										newdir + root.replace(copy_from, ''))
							except:
								pass
			for root, dirs, files in os.walk(newdir):
				for name in dirs:
					if os.listdir(os.path.join(root, name)) == []:
						os.rmdir(os.path.join(root, name))
		set_text_progressbar1(_('Complete'))
		del dic
		clean_tmp(self.tmpdir)
		progressbar2=glade.get_widget('progressbar2')
		gtk.gdk.threads_enter()
		progressbar2.set_fraction(1)
		preference.progressbar2.set_fraction(1)
		gtk.gdk.threads_leave()


class Conversation(threading.Thread):

	def __init__(self):
		threading.Thread.__init__(self)
		Conversation.value = None

	def run(self):
		treeview = glade.get_widget('job queue')
		model = treeview.get_model()
		while len(model) > 0:
			start()
			if Conversation.value == (_('Interrupted')):
				print Conversation.value
				break
		gtk.gdk.threads_enter()
		glade.get_widget('progressbar2').set_text('  ')
		preference.progressbar2.set_text('  ')
		glade.get_widget('stop_button').set_sensitive(False)
		gtk.gdk.threads_leave()

	def stop(self):
		return Conversation.pid



#!/usr/bin/env python
import sys
import os

if sys.version < '2.2':
	sys.exit('Error: Python-2.2 or newer is required. Current version:\n %s'
			% sys.version)

try:
	import gtk
	import gtk.glade
except:
	sys.exit('Error: gtk and gtk.glade are required')

try:
	import pygtk
	pygtk.require("2.0")
except:
	sys.exit('Error: pygtk 2.0 is required')



if not os.path.exists('/usr/share/gcue2tracks/'):
	os.system('sudo mkdir /usr/share/gcue2tracks/')

os.system('cp gCue2tracks.desktop /usr/share/applications')
os.system('cp gCue2tracks.png /usr/share/pixmaps')
os.system('cp gCue2tracks.glade /usr/share/gcue2tracks/gCue2tracks.glade')
os.system('cp gCue2tracks.png /usr/share/gcue2tracks')
os.system('cp gcue2tracks /usr/bin')
os.system('cp menu/gCue2tracks /usr/share/menu')
os.system('cp gCue2tracks.py /usr/share/gcue2tracks/gCue2tracks.py')
os.system('cp config.py /usr/share/gcue2tracks')
os.system('cp decoder.py /usr/share/gcue2tracks')
os.system('cp preference.py /usr/share/gcue2tracks')







#!/usr/bin/python
#-*- coding: utf8 -*-

from config import *
import decoder as decoder
import gobject, gtk, gettext


_ = gettext.gettext
trayicon = gtk.StatusIcon()
trayicon.set_from_icon_name('gCue2tracks')
win = gtk.Window(gtk.WINDOW_POPUP)
screen = win.get_screen()
#label1 = gtk.Label(str='None')
progressbar1 = gtk.ProgressBar()
progressbar2 = gtk.ProgressBar()

def _on__expose_event(window, _event):
	w, h = window.size_request()
	window.style.paint_flat_box(window.window,
								gtk.STATE_NORMAL, gtk.SHADOW_OUT,
								None, window, "tooltip",
								0, 0, w, h)
	return False

''' create a popup window each time tooltip is requested '''
win.set_border_width(1)
win.set_resizable(False)
win.set_app_paintable(True)
win.set_name('gtk-tooltips')
win.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_TOOLTIP)

win.set_events(gtk.gdk.POINTER_MOTION_MASK)
outtertipbox = gtk.VBox()
outtertipbox.pack_start(progressbar1, False, False, 1)
outtertipbox.pack_start(progressbar2, False, False, 1)
win.add(outtertipbox)
win.connect('expose-event', _on__expose_event)
outtertipbox.show_all()
#win.set_size_request(300, -1)

def notebook(widget,a,b):
	gobject.timeout_add(10, lambda: Tree.get_widget('close').grab_focus())

def bsavep(widget):

	def dialogdestroy(widget):
		dialog.destroy()

	def bsaveok(widget):
		path = dialog.get_filename()
		if button == 'savefolder_preference_btn':
			Tree.get_widget('open_folder').set_text(path)
			set('open_folder', path)
		else:
			Tree.get_widget('tmp_dir').set_text(path)
			set('tmp_dir', path)
		dialog.destroy()

	button = widget.get_name()
	glade = gtk.glade.XML(decoder.pathglade, 'filechooserdialog1', 'gCue2tracks')
	dic = {'bsave_cancel' : dialogdestroy, 'bsave_ok' : bsaveok}
	glade.signal_autoconnect(dic)
	dialog = glade.get_widget('filechooserdialog1')
	dialog.set_current_folder(os.path.expanduser('~'))
	dialog.set_title(_('Select folder'))
	response = dialog.run()
	dialog.destroy()

def codec_btn(widget):
	def dialogdestroy(widget):
		dialog.destroy()
	def bsaveok(widget):
		path = dialog.get_filename()
		Tree.get_widget(button[:-3]).set_text(path)
		set('widget', path)
		dialog.destroy()
	button = widget.get_name()
	glade = gtk.glade.XML(decoder.pathglade, 'filechooserdialog1', 'gCue2tracks')
	dic = {'bsave_cancel' : dialogdestroy, 'bsave_ok' : bsaveok}
	glade.signal_autoconnect(dic)
	dialog = glade.get_widget('filechooserdialog1')
	dialog.set_current_folder(os.path.expanduser('~'))
	dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
	dialog.set_title(_('Select file'))
	response = dialog.run()
	dialog.destroy()

def destroy(widget):
	map(lambda x: set(x, Tree.get_widget(x).get_text()), widgets)
	widgets_ = [
		'filename_filter', 'mp3advance', 'mp3_utf',
		'convert_cue', 'copy_image', 'show_systray', 'show_in_taskbar',
		'minimize_on_close']
	map(lambda x:set(x, Tree.get_widget(x).get_active()), widgets_)
	window = Tree.get_widget('preferences_window')
	window.hide()

def mp3_utf (widget):
	mutagen = True
	try: import mutagen.id3
	except ImportError:mutagen = False
	if not mutagen and Tree.get_widget('mp3_utf').get_active():
		Tree.get_widget('mp3_utf').set_active(False)
		message = _('mid3v2 (from python-mutagen) is not found')
		decoder.show_dialog(message)

##### systray #################################################################

def calculate_pos(widget):
	MARGIN = 4
	if widget is not None:
		try:
			x, y = widget.window.get_origin()
			if widget.flags() & gtk.NO_WINDOW:
				x += widget.allocation.x
				y += widget.allocation.y
			height = widget.allocation.height
		except:
			_icon_screen, icon_rect, _icon_orient = widget.get_geometry()
			x = icon_rect[0]
			y = icon_rect[1]
			height = icon_rect[3]
	w, h = win.size_request()

	screen = win.get_screen()
	pointer_screen, px, py, _ = screen.get_display().get_pointer()
	if pointer_screen != screen:
		px = x
		py = y
	try:
		# Use the monitor that the systemtray icon is on
		monitor_num = screen.get_monitor_at_point(x, y)
	except:
		# No systemtray icon, use the monitor that the pointer is on
		monitor_num = screen.get_monitor_at_point(px, py)
	monitor = screen.get_monitor_geometry(monitor_num)

	try:
		# If the tooltip goes off the screen horizontally, realign it so that
		# it all displays.
		if (x + w) > monitor.x + monitor.width:
			x = monitor.x + monitor.width - w
		# If the tooltip goes off the screen vertically (i.e. the system tray
		# icon is on the bottom of the screen), realign the icon so that it
		# shows above the icon.
		if ((y + h + height + MARGIN) >
			monitor.y + monitor.height):
			y = y - h - MARGIN
		else:
			y = y + height + MARGIN
	except:
		pass
	try:
		return x, y
	except:
		#Fallback to top-left:
		return monitor.x, monitor.y

def tooltip_show_manually():
	if True:
		_pscreen, px, py, _mods = screen.get_display().get_pointer()
		_icon_screen, icon_rect, _icon_orient = trayicon.get_geometry()
		x = icon_rect[0]
		y = icon_rect[1]
		width = icon_rect[2]
		height = icon_rect[3]
		if px >= x and px <= x+width and py >= y and py <= y+height:
			if not win.get_property('visible'):
				#self.traytips._start_delay(self.trayicon)
				x, y = calculate_pos(trayicon)
				win.move(x, y)
				win.show_all()
		else:
			#self.traytips._remove_timer()
			win.hide()

def iterate_status_icon():
	if trayicon.get_visible():
		tooltip_show_manually()
	gobject.timeout_add(250, iterate_status_icon)
	
def preference_activate():
	global Tree, widgets
	Tree = decoder.gtk.glade.XML(decoder.pathglade,
								'preferences_window', 'gCue2tracks')
	'''restore dialog state'''
	widgets_ = [
			'filename_filter', 'mp3advance', 'mp3_utf',
			'convert_cue', 'copy_image', 'show_systray', 'show_in_taskbar',
			'minimize_on_close']
	map(lambda x:Tree.get_widget(x).set_active(get(x)), widgets_)

	widgets = [
		'tmp_dir', 'format', 'ogg', 'mp3', 'flac', 'mac',
		'wavpack', 'wvunpack', 'open_folder', 'codepage']
	map(lambda x:Tree.get_widget(x).set_text(get(x)), widgets)

	window = Tree.get_widget('preferences_window')
	dic = {
		'destroy' : destroy, 'bsavep' : bsavep,
		'tmp_dir' : bsavep, 'notebook' : notebook, 'codec_btn' : codec_btn,
		'mp3_utf' : mp3_utf, 'show_systray' : show_systray,
		'minimize_on_close' : minimize_on_close,
		'show_in_taskbar' : show_in_taskbar
			}
	Tree.get_widget('show_in_taskbar').set_sensitive(get('show_systray'))
	Tree.signal_autoconnect(dic)
	window.show()

def window_state_event_cb(window, event):
	if event.changed_mask & gtk.gdk.WINDOW_STATE_ICONIFIED:
		if event.new_window_state & gtk.gdk.WINDOW_STATE_ICONIFIED:
			#Window was minimized
			decoder.win.hide()
			return True

def make_menu(event_button, event_time, data=None):
	menu = gtk.Menu()
	agr = gtk.AccelGroup()
	decoder.win.add_accel_group(agr)
	close_item = gtk.ImageMenuItem(gtk.STOCK_QUIT, agr)
	key, mod = gtk.accelerator_parse("Q")
	close_item.add_accelerator("activate", agr, key, mod, gtk.ACCEL_VISIBLE)
	#Append the menu items
	menu.append(close_item)
	#add callbacks
	close_item.connect_object('activate', close_app, 'Quit')
	#Show the menu items
	close_item.show()
	#Popup the menu
	menu.popup(None, None, None, event_button, event_time)

def close_app(data=None):
	write()
	gtk.main_quit()

def on_right_click(data, event_button, event_time):
	make_menu(event_button, event_time)

def show_window(widget):
	if decoder.win.get_property('visible') and \
										(decoder.win.iconify_initially == 0):
		decoder.win.hide()
	else:
		decoder.win.show_all()
		decoder.win.deiconify()
	decoder.win.set_property('skip-taskbar-hint', get('show_in_taskbar'))
	return True

def hide_window(win, event):
	if not get('show_systray'):
		win.iconify()
	else:
		win.hide()
	return True

def show_systray(widget):
	is_show_systray = Tree.get_widget('show_systray').get_active()
	trayicon.set_visible(is_show_systray)
	if not is_show_systray:
		Tree.get_widget('show_in_taskbar').set_active(False)
	Tree.get_widget('show_in_taskbar').set_sensitive(is_show_systray)

def minimize_on_close(widget):
	decoder.win = glade.get_widget('mainWindow')
	if not Tree.get_widget('minimize_on_close').get_active():
		decoder.win.disconnect(decoder.handler_id_moc)
	else:
		decoder.handler_id_moc = decoder.win.connect('delete-event', hide_window)

def show_in_taskbar(widget):
	if not Tree.get_widget('show_in_taskbar').get_active():
			decoder.win.disconnect(decoder.handler_id)
	else:
		decoder.handler_id = decoder.win.connect('window-state-event',
													window_state_event_cb)
	decoder.win.set_property('skip-taskbar-hint', get('show_in_taskbar'))

def systray():
	decoder.win = glade.get_widget('mainWindow')
	if get('show_in_taskbar'):
		decoder.handler_id = decoder.win.connect('window-state-event',
													window_state_event_cb)
	trayicon.set_visible(get('show_systray'))
	trayicon.connect('activate', show_window)
	trayicon.connect('popup-menu', on_right_click)
	iterate_status_icon()

	if get('minimize_on_close'):
		decoder.handler_id_moc = decoder.win.connect('delete-event', hide_window)



#!/usr/bin/python
#-*- coding: utf8 -*-

import gtk
import gtk.glade
import threading
import subprocess
from string import find
import time
import sys
import gettext
import locale
import pango
from config import *
from decoder import *
import getopt
import gobject
import urllib
import re
import preference as preference

VERSION = '0.4.5'


def get_file_path_from_dnd_dropped_uri(uri):
	path = urllib.url2pathname(uri) # escape special chars
	path = path.strip('\r\n\x00') # remove \r\n and NULL
	# get the path to file
	if path.startswith('file://'): # nautilus, rox
		path = path[7:]
	elif path.startswith('file:'): # xffm
		path = path[5:]
	return path


def on_mainWindow_drag_data_received(
				widget, drag_context, x, y, selection_data, target_type, time):
	if target_type == 0:
		uri = selection_data.data.strip()
		uri_splitted = uri.split()
		thread = None
		for uri in uri_splitted:
			path = get_file_path_from_dnd_dropped_uri(uri)
			if os.path.isdir(path): # is it dir?
				thread = Searching_cue(thread=thread, folder_cue=path)
				thread.start()
			else:
				if path.endswith('.cue'):
					glade.get_widget('filename').set_text(path)
					gui.bloc = True
					gui.getcuetag(path)
					gui.add_task(widget)
					gui.bloc = False


def treeview_setup_dnd(treeview):
	"""Setup a treeview to move rows using drag and drop.  """
	target_entries = [('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),
						('text/plain', 0, 1),
						('TEXT', 0, 2),
						('STRING', 0, 3), ]
	treeview.enable_model_drag_source(
		gtk.gdk.BUTTON1_MASK, target_entries, gtk.gdk.ACTION_MOVE)
	treeview.enable_model_drag_dest(target_entries, gtk.gdk.ACTION_MOVE)
	treeview.connect('drag-data-received', drag_data_received_data)
	treeview.connect('drag_data_get', drag_data_get_data)
	treeview.connect('cursor-changed', on_selection_changed)


def drag_data_get_data(treeview, context, selection, target_id, etime):
	treeselection = treeview.get_selection()


def drag_data_received_data(treeview, context, x, y, selection, info, etime):
	model_to_copy, iter_to_copy = treeview.get_selection().get_selected_rows()
	model = treeview.get_model()
	drop_info = treeview.get_dest_row_at_pos(x, y)
	if drop_info:
		path, position = drop_info
		iter = model.get_iter(path)
		if model.iter_depth(iter)is not 0:
			return
		if (position == gtk.TREE_VIEW_DROP_BEFORE
			or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
			model_to_copy.move_before(model.get_iter(iter_to_copy[0]), iter)
		else:
			model_to_copy.move_after(model.get_iter(iter_to_copy[0]), iter)
	else:
		model_to_copy.move_after(model.get_iter(iter_to_copy[0]),
											model.get_iter(len(model)-1))
	for n in range(len(model)):
		model[n][0] = n + 1


def on_selection_changed(treeview):
	cursor_position = treeview.get_cursor()[0]
	if not cursor_position:
		return

	'''restore parametrs if cursor changes'''
	try:
		a = gui.bloc
	except:
		return
	gui.bloc = True
	dic = treeview.get_model()[cursor_position[0]][6]
	widget = [
		'quality', 'album', 'date', 'genre', 'composer',
		'bin_file', 'filename', 'performer']
	map(lambda x: glade.get_widget(x).set_text(dic[x]), widget)
	var = [
		'bitratebtn', 'VBR',
		'same_folder', 'qualitybtn']
	map(lambda x: glade.get_widget(x).set_active(dic[x]), var)
	glade.get_widget('level').set_active(dic['level_n'])
	glade.get_widget('codec').set_active(dic['codec_n'])
	glade.get_widget('bitrate').set_value(float(dic['bitrate']))
	glade.get_widget('savefolder').set_text(dic['folder'])
	gui.bloc = a

	if len(cursor_position) > 1:
		gui.renderer.set_property('editable', True)
	else:
		gui.renderer.set_property('editable', False)
		#if treeview.get_cursor()[1] is treeview.get_column(0):
			#if treeview.row_expanded(cursor_position):
				#treeview.collapse_row(cursor_position)
			#else:
				#treeview.expand_to_path(cursor_position)


def notebook(widget, a, b):
	gobject.timeout_add(60, lambda: glade.get_widget('start').grab_focus())


def changeg(widget):
	glade.get_widget('VBR').set_sensitive(
								glade.get_widget('bitratebtn').get_active())
	glade.get_widget('bitrate').set_sensitive(
								glade.get_widget('bitratebtn').get_active())
	glade.get_widget('quality').set_sensitive(
								glade.get_widget('qualitybtn').get_active())
	codecn = glade.get_widget('codec').get_active_text()
	if codecn not in ['mp3', 'ogg']:
		glade.get_widget('VBR').set_sensitive(False)
		glade.get_widget('bitrate').set_sensitive(False)
		glade.get_widget('quality').set_sensitive(False)
		return
	if codecn == 'mp3' and get('mp3advance'):
		glade.get_widget('VBR').set_sensitive(True)
		glade.get_widget('bitrate').set_sensitive(True)
		glade.get_widget('quality').set_sensitive(True)
	if codecn == 'ogg':
		glade.get_widget('bitrate').set_range(32, 500)
	else:
		glade.get_widget('bitrate').set_range(32, 320)


def show_log(widget):
	show = glade.get_widget('show_log').get_active()
	glade.get_widget('log').set_property('visible', show)
	glade.get_widget('mainWindow').resize(1, 1)
	set('show_log', show)


def job_down(widget):
	cursor_position = gui.treeview.get_cursor()[0][0]
	if cursor_position == len(gui.tree_store) - 1:
		return
	iter = gui.tree_store.get_iter(cursor_position)
	gui.tree_store.move_after(iter,
		gui.tree_store.get_iter(cursor_position + 1))
	for n in range(len(gui.tree_store)):
		gui.tree_store[n][0] = n + 1


def job_up(widget):
	cursor_position = gui.treeview.get_cursor()[0][0]
	if cursor_position == 0:
		return
	iter = gui.tree_store.get_iter(cursor_position)
	gui.tree_store.move_before(iter,
			gui.tree_store.get_iter(cursor_position - 1))
	for n in range(len(gui.tree_store)):
		gui.tree_store[n][0] = n + 1


def isutf(line):
	if get('convert_cue'):
		try:
			s = line.decode('utf-8')
		except:
			try:
				line = line.decode(get('codepage'))
			except:
				message = _('Bad codepage')
				print message
				try:
					print_log(message, False)
					dialog = gtk.MessageDialog(parent=None,
							flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING,
							buttons=gtk.BUTTONS_OK)
					dialog.set_markup(message)
					if len(threading.enumerate()) > 1:
						return ''
					response = dialog.run()
					dialog.destroy()
				except:
					pass
				line = '							  '
	return line


class Searching_cue(threading.Thread):

	def __init__(self, thread=None, folder_cue=None):
		threading.Thread.__init__(self)
		self.thread = thread
		self.folder_cue = folder_cue

	def run(self):
		if self.thread:
			self.thread.join()
		self.bloc = True
		glade.get_widget('find_cue').set_sensitive(False)
		treeview = glade.get_widget('job queue')
		tree_store = treeview.get_model()
		if self.folder_cue:
			folder_cue = self.folder_cue
		else:
			folder_cue = glade.get_widget('find_cue_folder').get_text()
		set('find_cue_folder', folder_cue)
		if not os.path.isdir(folder_cue):
			return
		quality = glade.get_widget('quality').get_text()
		bitrate = str(int(glade.get_widget('bitrate').get_value()))
		level = glade.get_widget('level').get_active_text()
		level_n = glade.get_widget('level').get_active()
		bitratebtn = glade.get_widget('bitratebtn').get_active()
		qualitybtn = glade.get_widget('qualitybtn').get_active()
		VBR = glade.get_widget('VBR').get_active()
		codec = glade.get_widget('codec').get_active_text()
		codec_n = glade.get_widget('codec').get_active()
		folder = glade.get_widget('savefolder').get_text()
		same_folder = glade.get_widget('same_folder').get_active()
		prog = re.compile(r'".+"')
		for root, dirs, files in os.walk(folder_cue):
			for name in files:
				if name.endswith('cue'):
					filename = os.path.join(root, name)
					GuiPart.folder = root
					if glade.get_widget('same_folder').get_active():
						folder = root
					'''get tag from cue file'''
					dic = {
		'performer': '', 'album': '', 'date': '', 'genre': '', 'composer': '',
		'bitrate': bitrate, 'quality': quality, 'filename': filename,
		'level': level, 'level_n': level_n, 'VBR': VBR, 'codec': codec,
		'bitratebtn': bitratebtn, 'qualitybtn': qualitybtn,
		'codec_n': codec_n, 'folder': folder, 'same_folder': same_folder,
					}
					f = open(filename, 'r')
					line = f.readline()#.replace('"', '')
					result = prog.findall(line)
					if result:
						result = isutf(result[0].replace('"', ''))
					gui.bin_filename = None
					track, filename, title, performers = [], [], [], []
					while line:
						if 'TRACK' in line and 'REM' not in line:
							track.append(line.strip()[6:8])
						if 'GENRE' in line:
							dic['genre'] = line[10:-2].replace('"', '')
						if 'DATE' in line:
							dic['date'] = line.split(' ')[2][:4]
						if 'PERFORMER' in line:
							dic['performer'] = result
							performers.append(result)
						if 'COMPOSER' in line:
							dic['composer'] = result
						if 'TITLE' in line:
							if line[0]==' ':
								title.append(result)
							else:
								dic['album'] = result
						if 'FILE' in line:

							# if multitrack
							seek = f.tell()
							line1 = f.readline()
							if 'TRACK' in line1:
								filename.append(isutf(line)[5:-2].replace(
											' WAVE', '').replace('"', ''))
							f.seek(seek)

							gui.bin_filename = (isutf(line)[5:-2]).replace(
													' WAVE', '').replace('"', '')
							full_path = os.path.join(root, gui.bin_filename)
							if not os.path.isfile(full_path.encode('utf-8')):
								gui.bin_filename = gui.find_bin(root)
								if not gui.bin_filename:
									gui.bin_filename = 'None'
									message = _('Bin file not found\n<b>%s</b>') %full_path
									gtk.gdk.threads_enter()
									print message.replace('<b>', '').replace('</b>', '')
									dialog = gtk.MessageDialog(parent=None,
											flags=gtk.DIALOG_MODAL, type=gtk.MESSAGE_WARNING,
											buttons=gtk.BUTTONS_OK)
									dialog.set_markup(message)
									gobject.timeout_add(10000, lambda: dialog.destroy())
									response = dialog.run()
									dialog.destroy()
									gtk.gdk.threads_leave()
							dic['bin_file'] = gui.bin_filename
						line = f.readline()
						result = prog.findall(line)
						if result:
							result = isutf(result[0].replace('"', ''))
					if gui.bin_filename is None:
						f.close()
						break
					f.close()
					'''get album length'''
					if len(filename) is not 1:
						dic['bin_files'] = filename
					if len(performers) is not 1:
						dic['performers'] = performers
						for n in range(len(performers)-1):
							if performers[n] != performers[n + 1]:
								dic['performer'] = _('Various artists')
					gtk.gdk.threads_enter()
					tree_store.append(
						None, (len(tree_store) + 1, '%s - %s' % (
						dic['performer'], dic['album']), '',
						True, None, None, dic))
					treeview.set_cursor(len(tree_store) - 1)
					iter = tree_store.get_iter(len(tree_store) - 1)
					n = 0
					for item in title:
						n += 1
						tree_store.append(iter,
							(track[n-1], item, None, True, '',
							treeview.get_style().base[gtk.STATE_SELECTED].to_string(), None))
					gtk.gdk.threads_leave()
					break
		gtk.gdk.threads_enter()
		glade.get_widget('del_task').set_sensitive(True)
		if len(tree_store) > 0:
			glade.get_widget('start').set_sensitive(True)
		glade.get_widget('start').grab_focus()
		glade.get_widget('find_cue').set_sensitive(True)
		if len(tree_store) > 1:
			glade.get_widget('job_up').set_sensitive(True)
			glade.get_widget('job_down').set_sensitive(True)
		gtk.gdk.threads_leave()
		print 'searching cue done'
		self.bloc = False


class Add_queue(threading.Thread):

	def __init__(self, job_name, dic):
		threading.Thread.__init__(self)
		self.job_name = job_name
		self.dic = dic

	def run(self):
		treeview = glade.get_widget('job queue')
		tree_store = treeview.get_model()
		gtk.gdk.threads_enter()
		tree_store.append(
					None, (len(tree_store)+1, self.job_name, '',
					True, None, None, self.dic))
		treeview.set_cursor(len(tree_store) - 1)
		if len(tree_store) == 2:
			glade.get_widget('job_up').set_sensitive(True)
			glade.get_widget('job_down').set_sensitive(True)
		if len(threading.enumerate()) > 2:
			progressbar2=glade.get_widget('progressbar2')
			text = '%s   in queue:%s' %(Conversation.job_name,len(tree_store))
			progressbar2.set_text(text)
			preference.progressbar2.set_text(text)
		gtk.gdk.threads_leave()
		"""
			get tags
		"""
		filename = glade.get_widget('filename').get_text()
		path = os.path.split(filename)[0]
		f = open(filename, 'r')
		track, filename, title , performers = [], [], [], []
		line = f.readline()
		prog = re.compile(r'".+"')
		result = prog.findall(line)
		if result:
			result = isutf(result[0].replace('"', ''))
		while line:
			if 'TITLE' in line and line[0]==' ':
				title.append(result)
			line = line.replace('"', "")
			if 'TRACK' in line and 'REM' not in line:
				track.append(line.strip()[6:8])
			if 'FILE' in line:
				seek = f.tell()
				line1 = f.readline()
				if 'TRACK' in line1:
					filename.append(isutf(line)[5:-2].replace(' WAVE', ''))
				f.seek(seek)
			if 'PERFORMER' in line:
				performers.append(result)
			line = f.readline()
			result = prog.findall(line)
			if result:
				result = isutf(result[0].replace('"', ''))
		f.close()
		"""
			get album length
		"""
		n = 0
		if len(filename) == 1:
			track_light1, light = track_light(
						path, glade.get_widget('bin_file').get_text(), glade)
			iter = tree_store.get_iter(len(tree_store)-1)
			gtk.gdk.threads_enter()
			for item in title:
				n += 1
				tree_store.append(iter,
					(track[n-1], item, track_light1[n-1], True, '',
					treeview.get_style().base[gtk.STATE_SELECTED].to_string(), None))
			tree_store[iter][2] = light
			gtk.gdk.threads_leave()
		else:
			iter = tree_store.get_iter(len(tree_store)-1)
			gtk.gdk.threads_enter()
			for item in title:
				n += 1
				tree_store.append(iter,
					(track[n-1], item, '', True, '',
					treeview.get_style().base[gtk.STATE_SELECTED].to_string(), None))
			glade.get_widget('bin_file').set_text('tracks')
			gtk.gdk.threads_leave()
			dic = tree_store[iter][6]
			dic.update(bin_files = filename)
			tree_store[iter][6] = dic
		if len(performers) is not 1:
			self.dic['performers'] = performers
			for n in range(len(performers)-1):
				if performers[n] != performers[n + 1]:
					glade.get_widget('performer').set_text(_('Various artists'))

class GuiPart:

	def __init__(self):

		def insert_one_tag_into_buffer(buffer, name, *params):
			tag = gtk.TextTag(name)
			while(params):
				tag.set_property(params[0], params[1])
				params = params[2:]
			table = buffer.get_tag_table()
			table.add(tag)

		self.bloc = True
		dic = {'on_mainWindow_destroy': self.endApplication,
				'onf': self.onf, 'start_recomp': self.start_recomp,
				'bsave': self.bsave, 'stop': self.stop,
				'on_about1_activate': self.on_about_activate,
				'codec_changed': self.codec_changed,
				'changeg': changeg, 'sync': self.sync,
				'show_log': show_log, 'note-book': notebook,
				'on_preference_activate': on_preference_activate,
				'add_task': self.add_task, 'del_task': self.del_task,
				'job_down': job_down, 'job_up': job_up,
				'find_cue': self.find_cue, 'find_cue_dialog': self.find_cue_dialog,
				'on_mainWindow_drag_data_received': on_mainWindow_drag_data_received}
		glade.signal_autoconnect(dic)
		del dic
		bitrait = glade.get_widget('bitrate')
		'''read config'''
		bitrait.set_value(get('bitrait'))
		glade.get_widget('quality').set_text(get('quality'))
		glade.get_widget('savefolder').set_text(get('savepath'))
		glade.get_widget('find_cue_folder').set_text(get('find_cue_folder'))
		widget = [
			'same_folder', 'show_log', 'level', 'qualitybtn', 'VBR']
		map(lambda x: set(x, glade.get_widget(x).set_active(get(x))), widget)
		codecs_list(glade)
		if glade.get_widget('codec').get_active_text() == 'ogg':
			bitrait.set_range(32, 500)
		GuiPart.textview = glade.get_widget('textview1')
		buffer=GuiPart.textview.get_buffer()
		insert_one_tag_into_buffer(buffer, 'bold', 'weight', pango.WEIGHT_BOLD)
		glade.get_widget('log').set_property('visible', get('show_log'))
		win = glade.get_widget('mainWindow')
		win.set_property('skip-taskbar-hint', get('show_in_taskbar'))
		win.drag_dest_set(gtk.DEST_DEFAULT_ALL,
								[('text/uri-list', 0, 0)], gtk.gdk.ACTION_MOVE)
		#win.reshow_with_initial_size()
		'''first build'''
		self.treeview = glade.get_widget('job queue')
		self.treeview.set_rules_hint(True)
		self.treeview.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
		self.tree_store = self.treeview.get_model()
		self.tree_store = gtk.TreeStore(str, str, str, 'gboolean', str, str,
														gobject.TYPE_PYOBJECT)

		TARGETS = [
					('MY_TREE_MODEL_ROW', gtk.TARGET_SAME_WIDGET, 0),
					('text/plain', 0, 1),
					('TEXT', 0, 2),
					('STRING', 0, 3),
					]

		self.treeview.set_model(self.tree_store)
		treeview_setup_dnd(self.treeview)
		renderer0 = gtk.CellRendererText()
		column = gtk.TreeViewColumn('', renderer0, text=0)
		self.renderer = gtk.CellRendererText()
		self.renderer.set_properties(
								xalign=0, editable=True)
		self.renderer.connect('edited', col0_edited_cb, self.tree_store)
		self.renderer1 = gtk.CellRendererToggle()
		self.renderer1.set_properties(
							xalign=1.0, activatable=True)
		self.renderer1.connect('toggled', col1_toggled_cb, self.tree_store)
		column1 = gtk.TreeViewColumn('', self.renderer1)
		column1.add_attribute(self.renderer1, 'active', 3)
		column1.set_clickable(True)
		column2 = gtk.TreeViewColumn(_('Track length'), renderer0, text=2)
		column0 = gtk.TreeViewColumn(_('Track name'), self.renderer, text=1)
		column0.add_attribute(self.renderer, 'foreground', 5)
		column.add_attribute(renderer0, 'foreground', 5)
		column0.set_resizable(True)
		column1.set_resizable(False)
		self.treeview.append_column(column)
		self.treeview.append_column(column0)
		self.treeview.append_column(column2)
		self.treeview.append_column(column1)
		#self.treeview.set_cursor(0)
		if ARGS:
			filename = ARGS[-1]
			if os.path.isdir(filename): # is it dir?
				thread = Searching_cue(thread=None, folder_cue=filename)
				thread.start()
				self.bloc = False
				return
			if not os.path.isfile(filename):
				print _('Wrong filename or folder'), filename
				self.bloc = False
				return
			path = os.path.split(filename)[0]
			glade.get_widget('filename').set_text(filename)
			glade.get_widget('start').set_sensitive(True)
			gtk.gdk.threads_enter()
			self.getcuetag(filename)
			gtk.gdk.threads_leave()
			self.add_task(self)
		self.bloc = False

	def on_about_activate(self, widget):
		"""
			about gcue2tracks
		"""
		Tree = gtk.glade.XML(pathglade, 'aboutdialog1', APP)
		about = Tree.get_widget('aboutdialog1')
		if  os.path.isfile('/usr/share/common-licenses/GPL-2'):
			fd = open('/usr/share/common-licenses/GPL-2')
			text = fd.read()
			about.set_license(text)
			fd.close()
		about.set_comments(
			'GTK+ Version: %s\nPyGTK Version: %s'
			% (self.tuple2str(gtk.gtk_version),
			self.tuple2str(gtk.pygtk_version)))
		about.set_website('http://trac-hg.assembla.com/gCue2tracks/wiki')
		about.set_version(VERSION)
		authors = []
		author = 'gCue2tracks: \n\tFomin Denis (fominde@gmail.com)'
		authors.append(author)
		authors.append(
		'\n%s\n\tOleg N. Stadnik (mail@lokee.rv.ua)\
		\n\tgoogle moogle\n\tEvgen' %_('THANKS:'))
		about.set_authors(authors)
		#about.set_artists('graf designer')
		for button in about.action_area.get_children():
			if button.get_property('label') == gtk.STOCK_CLOSE:
				button.connect('clicked', lambda x: about.hide())
		about.show()

	def tuple2str(self, tuple_):
		str_ = ''
		for num in tuple_:
			str_ += str(num) + '.'
		return str_[0:-1]

	def codec_changed(self, widget):
		if glade.get_widget('codec').get_active_text() == 'mp3':
			is_bitratebtn = glade.get_widget('bitratebtn').get_active()
			glade.get_widget('VBR').set_sensitive(is_bitratebtn)
			glade.get_widget('bitrate').set_sensitive(is_bitratebtn)
			is_qualitybtn = glade.get_widget('qualitybtn').get_active()
			glade.get_widget('qualitybtn').set_active(is_qualitybtn)
			glade.get_widget('quality').set_sensitive(is_qualitybtn)
			glade.get_widget('bitrate').set_range(32, 320)
			glade.get_widget('level').set_sensitive(False)
			glade.get_widget('bitratebtn').set_sensitive(True)
			glade.get_widget('qualitybtn').set_sensitive(True)
			if get('mp3advance'):
				glade.get_widget('quality').set_sensitive(True)
			return
		if glade.get_widget('codec').get_active_text() == 'ogg':
			glade.get_widget('qualitybtn').set_active(get('qualitybtn'))
			glade.get_widget('VBR').set_sensitive(
				not glade.get_widget('qualitybtn').get_active())
			glade.get_widget('bitrate').set_sensitive(
				not glade.get_widget('qualitybtn').get_active())
			glade.get_widget('level').set_sensitive(False)
			glade.get_widget('bitrate').set_range(32, 500)
			glade.get_widget('bitratebtn').set_sensitive(True)
			glade.get_widget('qualitybtn').set_sensitive(True)
			glade.get_widget('quality').set_sensitive(
				glade.get_widget('qualitybtn').get_active())
			return
		widget = [
			'VBR', 'bitrate', 'quality', 'bitratebtn', 'qualitybtn']
		map(lambda x: glade.get_widget(x).set_sensitive(False), widget)
		glade.get_widget('level').set_sensitive(True)

	def sync(self, widget):
		if not self.bloc:
			cursor = self.treeview.get_cursor()
			if len(glade.get_widget('job queue').get_model()) < 1:
				cursor = ((0, 0), 0)
				return
			treeview = glade.get_widget('job queue')
			model = treeview.get_model()
			dic = model[cursor[0]][6]
			dic.update(parametrs(glade).items())
			model[cursor[0]][6] = dic

	def find_cue_dialog(self, widget):
		dialog = gtk.FileChooserDialog(_('Select folder'), None,
						gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_OPEN, gtk.RESPONSE_OK))
		dialog.set_default_response(gtk.RESPONSE_OK)
		if not os.path.exists(get('open_folder')):
			set('open_folder', os.path.expanduser('~'))
		dialog.set_current_folder(get('open_folder'))
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			glade.get_widget('find_cue_folder').set_text(dialog.get_filename())
		dialog.destroy()

	def onf(self, widget):
		"""open file dialog"""
		dialog = gtk.FileChooserDialog(_('Open'), None,
						gtk.FILE_CHOOSER_ACTION_OPEN,
						(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
						gtk.STOCK_OPEN, gtk.RESPONSE_OK))

		dialog.set_default_response(gtk.RESPONSE_OK)
		if not os.path.exists(get('open_folder')):
			set('open_folder', os.path.expanduser('~'))
		dialog.set_current_folder(get('open_folder'))
		try:
			dialog.set_icon_from_file('/usr/share/pixmaps/gCue2tracks.png')
		except:
			pass
		filter = gtk.FileFilter()
		filter.set_name('Cue')
		filter.add_pattern('*.cue')
		dialog.add_filter(filter)
		filter = gtk.FileFilter()
		filter.set_name(_('All files'))
		filter.add_pattern('*')
		dialog.add_filter(filter)
		response = dialog.run()
		if response == gtk.RESPONSE_OK:
			self.bloc = True
			self.filename = dialog.get_filename()
			glade.get_widget('filename').set_text(self.filename)
			if glade.get_widget('same_folder').get_active():
				glade.get_widget('savefolder').set_text(dialog.get_current_folder())
			self.getcuetag()
			self.bloc = False
			if glade.get_widget('bin_file').get_text() is '':
				dialog.destroy()
				message = _('Bin file not found\n<b>%s</b>') %''
				show_dialog(message)
				return
			self.add_task(self)
			glade.get_widget('start').set_sensitive(True)
			glade.get_widget('start').grab_focus()
		dialog.destroy()

	def getcuetag(self, filename = None):
		'''get tag from cue file'''
		if filename is None:
			if not os.path.isfile(self.filename):
				return
			path = os.path.split(self.filename)[0]
			f = open(self.filename, 'r')
		else:
			path = os.path.split(filename)[0]
			f = open(filename, 'r')
			if glade.get_widget('same_folder').get_active():
				glade.get_widget('savefolder').set_text(path)
		fields = ['genre', 'date', 'performer', 'album', 'composer']
		map(lambda x: glade.get_widget(x).set_text(''), fields)
		line = f.readline()#.replace('"','')
		prog = re.compile(r'".+"')
		result = prog.findall(line)
		if result:
			result = isutf(result[0].replace('"', ''))
		while line:
			if 'TRACK' in line:
				break
			if 'GENRE' in line:
				glade.get_widget('genre').set_text(line[10:-2].replace('"', ''))
			if 'DATE' in line:
				glade.get_widget('date').set_text(line.split(' ')[2][:4])
			if 'PERFORMER' in line:
				glade.get_widget('performer').set_text(result)
			if 'COMPOSER' in line:
				glade.get_widget('composer').set_text(result)
			if 'TITLE' in line:
				glade.get_widget('album').set_text(result)
			if 'FILE' in line:
				a = (isutf(line)[5:-2]).replace(' WAVE', '')
				full_path = os.path.join(path, a)
				if not os.path.isfile(full_path.encode('utf-8')):
					a = self.find_bin(path)
					if not a:
						a = 'None'
						message = _('Bin file not found\n<b>%s</b>') %full_path
						show_dialog(message)
				glade.get_widget('bin_file').set_text(a)
			line = f.readline()#.replace('"','')
			result = prog.findall(line)
			if result:
				result = isutf(result[0].replace('"', ''))
		f.close()

	def find_cue(self, widget):
		thread = Searching_cue()
		thread.start()

	def find_bin(self, path):
		extention = ('ape', 'flac', 'wv', 'flake', 'wav', 'tta')
		for ext in extention:
			if not os.path.isdir(path):
				return
			for name in os.listdir(path):
				path_name = os.path.join(path, name)
				if path_name.endswith(ext) or not ext:
					path_name = os.path.split(path_name)
					return path_name[-1].decode('utf8')

	def bsave(self, widget):

		def dialogdestroy(widget):
			dialog.destroy()

		def bsaveok(widget):
			path = dialog.get_filename().decode(sys.getfilesystemencoding())
			glade.get_widget('savefolder').set_text(path)
			set('save_folder', path)
			dialog.destroy()

		Tree = gtk.glade.XML(pathglade, 'filechooserdialog1', APP)
		dic = {'bsave_cancel': dialogdestroy, 'bsave_ok': bsaveok}
		Tree.signal_autoconnect(dic)
		dialog = Tree.get_widget('filechooserdialog1')
		if not os.path.exists(get('open_folder')):
			set('save_folder', os.path.expanduser('~'))
		dialog.set_current_folder(get('save_folder'))
		response = dialog.show()

	def start_recomp(self, widget):
		self.testmode = 0
		self.start_recomp1(self)

	def add_task(self, widget):
		dic = parametrs(glade)
		if len(threading.enumerate()) == 1:
			glade.get_widget('start').set_sensitive(True)
		self.add_queue = Add_queue('%s - %s' % (dic['performer'], dic['album']), dic)
		self.add_queue.start()
		glade.get_widget('del_task').set_sensitive(True)

	def del_task(self, widget):
		selection = self.treeview.get_selection()
		model, selected = selection.get_selected_rows()
		iters = [model.get_iter(path) for path in selected]
		cursor_position = self.treeview.get_cursor()[0]
		if len(cursor_position) > 1:
			return
		if len(model) == 1:
			glade.get_widget('del_task').set_sensitive(False)
		if iters:
			for iter in iters:
				model.remove(iter)
			for n in range(len(model)):
				model[n][0] = n + 1
			if len(model) != 0:
				self.treeview.set_cursor(0)
			if len(model) == 1:
				glade.get_widget('job_up').set_sensitive(False)
				glade.get_widget('job_down').set_sensitive(False)
		if len(model) == 0:
			glade.get_widget('start').set_sensitive(False)
		if len(threading.enumerate()) > 1:
			progressbar2=glade.get_widget('progressbar2')
			text = '%s   in queue:%s' %(Conversation.job_name, len(model))
			progressbar2.set_text(text)
			preference.progressbar2.set_text(text)

	def start_recomp1(self, widget):
		glade.get_widget('start').set_sensitive(False)
		start_thread = Conversation()
		start_thread.start()

	def stop(self, widget):
		glade.get_widget('stop_button').set_sensitive(False)
		treads = threading.enumerate()
		a = treads[1].stop()
		Conversation.value = (_('Interrupted'))
		if a:
			try:
				os.kill(a, 9)
			except:
				pass
		if len(glade.get_widget('job queue').get_model()) > 0:
			glade.get_widget('start').set_sensitive(True)

	def endApplication(self, widget):
		if len(threading.enumerate()) > 1:
			self.stop(None)
			clean_tmp(get('tmpdir'))
		widget = [
			'same_folder', 'codec', 'level', 'bitratebtn', 'qualitybtn', 'VBR']
		map(lambda x: set(x, glade.get_widget(x).get_active()), widget)
		set('savepath', glade.get_widget('savefolder').get_text())
		set('quality', glade.get_widget('quality').get_text())
		set('bitrait', glade.get_widget('bitrate').get_value())
		write()
		gtk.main_quit()


gtk.gdk.threads_init()
APP = 'gCue2tracks'
DIR = '/usr/share/locale'
locale.setlocale(locale.LC_ALL, '')

for module in (gettext, gtk.glade):
	module.bindtextdomain(APP, DIR)
	module.textdomain(APP)

#gettext.install(APP, DIR, unicode = True)
_ = gettext.gettext

if os.path.dirname(sys.argv[0]) != '/usr/bin':
	workdir = os.getcwd()
else:
	workdir = '/usr/share/gcue2tracks'

pathglade = os.path.join(workdir, 'gCue2tracks.glade')
glade = gtk.glade.XML(pathglade, 'mainWindow', APP)
QUIET = None

longargs = 'help quiet output'
try:
	opts, args = getopt.getopt(sys.argv[1:], 'hqo:', longargs.split())
	ARGS = args
except getopt.GetoptError, err:
		print str(err)
		sys.exit(2)

for o, a in opts:
	if o in ('-h', '--help'):
		print 'gCue2tracks [--help] [--quiet] inputfile [...]'
		sys.exit()
	if o in ('-q', '--quiet'):
		print 'will be done later :-)'
		QUIET = True
	if o in ('-o', '--output'):
		print a


gui = GuiPart()
varc(glade, pathglade, QUIET)
gtk.gdk.threads_enter()
preference.glade = glade
preference.systray()
gtk.main()
gtk.gdk.threads_leave()



#!/usr/bin/python
#-*- coding: utf8 -*-
import os
try: # rename the process name to gCue2tracks
	import dl
	libc = dl.open('/lib/libc.so.6')
	libc.call('prctl', 15, 'gCue2tracks\0', 0, 0, 0)
except:
	pass

(
OPT_TYPE,
OPT_VAL,
OPT_DESC,
OPT_RESTART,
) = range(4)

opt_int = [ 'integer', 0 ]
opt_str = [ 'string', 0 ]
opt_bool = [ 'boolean', 0 ]
__options = {
	# name: [ type, default_value, help_string ]
	'savepath': [ opt_str, '' ],
	'codepage': [ opt_str, '' ],
	'mp3_utf': [ opt_bool, False ],
	'convert_cue': [ opt_bool, True ],
	'bitrait': [ opt_int, 128],
	'VBR': [ opt_bool, True ],
	'quality': [ opt_str, "7"],
	'qualitybtn': [ opt_bool, True ],
	'bitratebtn': [ opt_bool, False ],
	'same_folder': [ opt_bool, True ],
	'codec': [ opt_int, 0 ],
	'level': [ opt_int, 0 ],
	'format': [ opt_str, '/%P/%D - %A/%N - %t' ],
	'open_folder': [ opt_str, os.path.expanduser('~') ],
	'save_folder': [ opt_str, os.path.expanduser('~') ],
	'show_log': [ opt_bool, True ],
	'tmp_dir': [ opt_str, '/tmp' ],
	'ogg': [ opt_str, '/usr/bin/oggenc' ],
	'mp3': [ opt_str, '/usr/bin/lame' ],
	'mac': [ opt_str, '/usr/bin/mac' ],
	'flac': [ opt_str, '/usr/bin/flac' ],
	'wavpack': [ opt_str, '/usr/bin/wavpack' ],
	'wvunpack': [ opt_str, '/usr/bin/wvunpack' ],
	'filename_filter': [ opt_bool, False ],
	'mp3advance': [ opt_bool, False ],
	'disable_taging': [ opt_bool, False ],
	'tmpdir': [ opt_str, '/tmpfolder' ],
	'find_cue_folder' : [ opt_str, '' ],
	'copy_image' : [ opt_bool, False ],
	'show_systray': [ opt_bool, False ],
	'minimize_on_close': [ opt_bool, False ],
	'show_in_taskbar': [ opt_bool, False ],
}

def is_valid_int(val):
	try:
		ival = int(val)
	except:
		return None
	return ival

def is_valid_bool(val):
	if val == 'True':
		return True
	elif val == 'False':
		return False
	else:
		ival = is_valid_int(val)
		if ival:
			return True
		elif ival is None:
			return None
		return False
	return None

def is_valid_string(val):
	return val

def is_valid(type, val):
	if not type:
		return None
	if type[0] == 'boolean':
		return is_valid_bool(val)
	elif type[0] == 'integer':
		return is_valid_int(val)
	elif type[0] == 'string':
		return is_valid_string(val)
	else:
		return None

def write_line( fd, opt, value):
	if value is None:
		return
	value = value[1]
	 #convert to utf8 before writing to file if needed
	if isinstance(value, unicode):
		value = value.encode('utf-8')
	else:
		value = str(value)
	if isinstance(opt, unicode):
		opt = opt.encode('utf-8')
	s = ''
	s += opt
	fd.write(s + ' = ' + str(value) + '\n')

def write():
	(base_dir, filename) = os.path.split(conf_path + "/config")
	__tempfile = os.path.join(base_dir, '.' + filename)
	try:
		f = open(__tempfile, 'w')
	except IOError, e:
		return str(e)
	try:
		foreach(write_line, f)
	except IOError, e:
		return str(e)
	f.close()
	if os.path.exists(conf_path + "/config"):
		# win32 needs this
		try:
			os.remove(conf_path + "/config")
		except:
			pass
	try:
		os.rename(__tempfile, conf_path + "/config")
	except IOError, e:
		return str(e)
	os.chmod(conf_path + "/config", 0600)

def foreach( cb, data = None):
	for opt in __options:
		cb(data, opt, __options[opt])

def get(optname = None):
	if not optname:
		return __options.keys()
	if not __options.has_key(optname):
		return None
	return __options[optname][OPT_VAL]

def add_per(self, typename, name): # per_group_of_option
	if not self.__options_per_key.has_key(typename):
		return

	opt = self.__options_per_key[typename]
	if opt[1].has_key(name):
		# we already have added group name before
		return 'you already have added %s before' % name
	opt[1][name] = copy.deepcopy(opt[0])

def del_per(self, typename, name, subname = None): # per_group_of_option
	if not self.__options_per_key.has_key(typename):
		return

	opt = self.__options_per_key[typename]
	if subname is None:
		del opt[1][name]
	elif opt[1][name].has_key(subname):
		del opt[1][name][subname]

def set_per(self, optname, key, subname, value): # per_group_of_option
	if not self.__options_per_key.has_key(optname):
		return
	dict = self.__options_per_key[optname][1]
	if not dict.has_key(key):
		return
	obj = dict[key]
	if not obj.has_key(subname):
		return
	subobj = obj[subname]
	value = self.is_valid(subobj[OPT_TYPE], value)
	if value is None:
		return
	subobj[OPT_VAL] = value

def set(optname, value):
	if not __options.has_key(optname):
		return
	opt = __options[optname]
	value = is_valid(opt[OPT_TYPE], value)
	if value is None:
		return

	opt[OPT_VAL] = value

def read_line(line):
	index = line.find(' = ')
	var_str = line[0:index]
	value_str = line[index + 3:-1]
	set(var_str, value_str)


conf_path = os.path.expanduser('~') + "/.config/gCue2tracks"
if not os.path.isfile(conf_path + "/config"):
	if not os.path.isdir(conf_path):
		os.makedirs(conf_path)
	fd = open(conf_path + "/config", 'w')
	fd.close()
	print "create config" ####
try:
	fd = open(conf_path + "/config")
except:
	if os.path.exists(conf_path + "/config"):
		#we talk about a file
		print _('error: cannot open %s for reading') % conf_path + "/config"
for line in fd.readlines():
	# read config
	try:
		line = line.decode('utf-8')
	except UnicodeDecodeError:
		line = line.decode(locale.getpreferredencoding())
	read_line(line)
fd.close()
